/*
 *  
 *  Copyright (c) 2004 Steve Slaven, All Rights Reserved.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA 02111-1307 USA
 *  
*/
// This includes the map data handler, also handles caching and stuff like that
package net.hoopajoo.cheeselab3;
import java.io.File;
import java.util.*;
import jdbm.JDBMHashtable;
import jdbm.JDBMRecordManager;

class CheeseMapData {
	JDBMRecordManager db = null;
	JDBMHashtable rooms = null;
	//JDBMHashtable players = null;
	HashMap players = new HashMap();
	
	// This is the highest seen mtime, not the last saved one
	long highest_mtime = 0;
	
	CheeseMapData() {
		CheeseLab.dmesg( 2, "Creating map data instance" );
		initcache();
	}

	// this is the last saved highest mtime
	public long get_highest_mtime() {
		// get highest mtime from hash
		long max = Long.MAX_VALUE; // in case of exception do not refresh
		try {
			Long tmp_mtime = (Long)rooms.get( "highest_mtime" );
			if( tmp_mtime != null ) {
				CheeseLab.dmesg( 2, "Got last highest mtime: " + tmp_mtime );
				max = tmp_mtime.longValue();
			}else{
				max = 0;
			}
		}catch( Exception e ) {
			e.printStackTrace();
		}
		
		return( max );
	}
	
	public void initcache() {
		try {
			File cache = new File( CheeseMap.cachedir + "/tiles" );
			cache.mkdirs();
			
			db = new JDBMRecordManager( CheeseMap.cachedir + "/rooms.cache" );
			rooms = db.getHashtable( "rooms" );
			//players = db.getHashtable( "players" );			
		}catch( Exception e ) {
			CheeseLab.die( this, 1, "Unable to create room cache" );
		}		
	}
	
	public void update_room( byte[] r ) {
		CheeseLab.dmesg( 2, "Updating room" );

		CheeseRoomData room = (CheeseRoomData)CheeseDecoder.decode_object( r );
		CheeseLab.dmesg( 2, "Decoded room: " + room.roomid + " - " +
			room.x + "," + room.y + "," + room.z + " - " + room.mtime );
		/*
		HashMap rh = CheeseDecoder.decode_hash( r );
		CheeseRoomData room = new CheeseRoomData();

		room.roomid = Integer.parseInt( (String)rh.get( "roomid" ) );
		room.name = (String)rh.get( "name" );
		room.notes = (String)rh.get( "notes" );
		room.terrain = (String)rh.get( "terrain" );
		room.x = Integer.parseInt( (String)rh.get( "x" ) );
		room.y = Integer.parseInt( (String)rh.get( "y" ) );
		room.z = Integer.parseInt( (String)rh.get( "z" ) );

		room.exit_e = Integer.parseInt( (String)rh.get( "exit_e" ) );
		room.door_e = (String)rh.get( "exit_e_door" );
		room.exit_w = Integer.parseInt( (String)rh.get( "exit_w" ) );
		room.door_w = (String)rh.get( "exit_w_door" );
		room.exit_n = Integer.parseInt( (String)rh.get( "exit_n" ) );
		room.door_n = (String)rh.get( "exit_n_door" );
		room.exit_s = Integer.parseInt( (String)rh.get( "exit_s" ) );
		room.door_s = (String)rh.get( "exit_s_door" );
		room.exit_u = Integer.parseInt( (String)rh.get( "exit_u" ) );
		room.door_u = (String)rh.get( "exit_u_door" );
		room.exit_d = Integer.parseInt( (String)rh.get( "exit_d" ) );
		room.door_d = (String)rh.get( "exit_d_door" );

		String overlays = (String)rh.get( "overlays" );
		room.overlays = overlays.split( "," );
		*/
		
		try {
			rooms.put( new Integer( room.roomid ), room );
			
			if( room.mtime > highest_mtime ) {
				highest_mtime = room.mtime;
			}			
		}catch( Exception e ) {
			CheeseLab.dmesg( 0, "Error saving object room: " + e );
			e.printStackTrace();
		}
	}

	public void update_player( byte[] p ) {
		CheeseLab.dmesg( 2, "Updating player" );

		CheesePlayerData player = (CheesePlayerData)CheeseDecoder.decode_object( p );
		
		/*
		HashMap ph = CheeseDecoder.decode_hash( p );

		CheesePlayerData player = new CheesePlayerData();

		player.name = (String)ph.get( "userid" );

		player.x = Integer.parseInt( (String)ph.get( "x" ) );
		player.y = Integer.parseInt( (String)ph.get( "y" ) );
		player.z = Integer.parseInt( (String)ph.get( "z" ) );

		player.hit_max = Integer.parseInt( (String)ph.get( "hits_max" ) );
		player.hit_curr = Integer.parseInt( (String)ph.get( "hits" ) );
		player.move_max = Integer.parseInt( (String)ph.get( "moves_max" ) );
		player.move_curr = Integer.parseInt( (String)ph.get( "moves" ) );
		player.mana_max = Integer.parseInt( (String)ph.get( "mana_max" ) );
		player.mana_curr = Integer.parseInt( (String)ph.get( "mana" ) );
		*/
		
		try {
			CheeseLab.dmesg( 3, "Saving player: " + player.name );
			//players.remove( player.name );
			players.put( player.name, player );
		}catch( Exception e ) {
			CheeseLab.dmesg( 0, "Error saving player object player: " + e );
			e.printStackTrace();
		}
	}

	public CheesePlayerData[] get_players() {
		Vector data = new Vector();
		try {
			//JDBMEnumeration values = players.values();
			Iterator values = players.values().iterator();
			while( values.hasNext() )
				data.add( values.next() );
		
			//while( values.hasMoreElements() )
			//	data.add( values.nextElement() );
		}catch( Exception e ) {
			CheeseLab.dmesg( 0, "Exception in get_players: " + e );
			e.printStackTrace();
		}
		return( (CheesePlayerData[])data.toArray( new CheesePlayerData[0] ) );
	}

	public CheesePlayerData get_player( String name ) {
		try {
			return( (CheesePlayerData)players.get( name ) );
		}catch( Exception e ) {
			return( null );
		}
	}

	// Give mtime as the servers latest mtime, if our room is
	// at least that new we will not even bother with a round-trip
	public CheeseRoomData get_room( int id ) {
		return( get_room( id, 1 ) );
	}
	
	public CheeseRoomData get_room( int id, long mtime ) {
		try {
			CheeseRoomData room = (CheeseRoomData)rooms.get( new Integer(id) );
			
			if( room == null ) {
				// Send back a dummy room
				room = new CheeseRoomData();
				room.name = "Uncached Room (loading)";
				room.terrain = "?";
				room.overlays = new String[0];
				room.mtime = 0;
			}

			if( room.mtime < mtime ) {
				CheeseLab.dmesg( 2, "Requesting room " + room.roomid + " (" + room.mtime + "<" + mtime + ")" );
				// Request it from the server, a refresh should
				// be issued when it comes in sooo :p
				// you ought to get a "filling in" effect while spamming about
				CheeseMap.send_tx( 1, "get", id + ":" + room.mtime );
			}

			return( room );
		}catch( Exception e ) {
			return( null );
		}
	}
	
	public void save_max_mtime() {
		try {
			rooms.put( "highest_mtime", new Long( highest_mtime ) );
		}catch( Exception e ) {
			e.printStackTrace();
		}
	}
}
